==> snips-sam-0.0.33.tgz: package/src/session/consoleClient.ts <==
import { timeout, samConfigDirectory } from '../utils'
import wretch from 'wretch'
import { Wretcher } from 'wretch/dist/wretcher'
import { cli } from '../cli'
import { User, ConsoleToken, Assistant, AssistantStatus, ASRTrainingStatus } from '../models'

const path = require('path')
const fs = require('fs')
const uuidv4 = require('uuid/v4')

export class ConsoleClient {
    private wretch: Wretcher

    constructor() {
        // const httpsProxyAgent = require('https-proxy-agent')
        this.wretch = wretch()
        .url('https://external-gateway.snips.ai')
        // .options({ agent: httpsProxyAgent('http://127.0.0.1:8888') })
        .polyfills({
            fetch: require('node-fetch'),
            FormData: require('form-data'),
            URLSearchParams: require('url').URLSearchParams,
        })
        .catcher(401, _ => {
            throw new Error('You need to login again')
        })

        if (cli.config.consoleToken && cli.config.consoleToken.token !== undefined) {
            this.wretch.defaults({ headers: { Authorization: cli.config.consoleToken.token } })
        }
    }

    public async login(email: string, password: string): Promise<User> {
        return this.wretch
        .url('/v1/user/auth')
        .json({ email, password })
        .post()
        .res()
        .then(response => {
            const token = response.headers.get('authorization')
            if (token === null) throw new Error('Error fetching JWT token')
            return response.json()
            .then(json => {
                const user: User = new User(json['user'])
                return { user, token }
            })
        })
        .then(tuple => {
            return this.accessToken(tuple.user.id, tuple.token)
            .then(tokenObject => {
                const consoleCredentials = new ConsoleToken(tokenObject)
                consoleCredentials.token = 'JWT ' + consoleCredentials.token
                this.wretch.defaults({ headers: { Authorization: consoleCredentials.token } })
                return { user: tuple.user, token: consoleCredentials }
            })
        })
        .then(tuple => {
            const user: User = { id: tuple.user.id }
            cli.config.user = user
            cli.config.consoleToken = tuple.token
            return cli.config.save().then(_ => user)
        })
    }

    public async logout(userId: string, alias: string) {
        return this.wretch
        .url(`/v1/user/${userId}/accesstoken/${alias}`)
        .delete()
        .error(204, () => {})
        .res()
        .then(_ => {
            cli.config.user = undefined
            cli.config.consoleToken = undefined
            return cli.config.save()
        })
    }

    public async accessToken(userId: string, jwtToken: string): Promise<ConsoleToken> {
        const alias = (`sam-` + uuidv4()).replace(/-/g, '').substring(0, 29)
        return this.wretch
        .url(`/v1/user/${userId}/accesstoken`)
        .auth(jwtToken)
        .json({ alias })
        .post()
        .json(json => new ConsoleToken(json['token']))
    }

    public async assistantList(userId: string): Promise<Assistant[]> {
        return this.wretch
        .url('/v3/assistant')
        .query({ userId })
        .get()
        .json(json => {
            const assistants: Assistant[] = json['assistants'].map(assistant => Assistant.fromJSON(assistant))
            if (assistants === null || assistants === undefined) return []
            return assistants
        })
    }

    public async assistantStatus(assistantId: string): Promise<AssistantStatus> {
        return this.wretch
        .url(`/v3/assistant/${assistantId}/status`)
        .get()
        .json(json => new AssistantStatus(json))
    }

    public async nluTraining(assistantId: string, forceTraining = false) {
        let needTraining = false
        await this.assistantStatus(assistantId)
        .then(status => needTraining = status.needTraining)
        .catch(error => {
            if (error.statusCode === 409) { // Already training
                needTraining = false
            } else {
                throw new Error('NLU training error: ' + error.message)
            }
        })

        if (needTraining || forceTraining) {
            await this.wretch.url('/v1/training').json({ assistantId }).post()
        }

        while (needTraining) {
            await timeout(2000)
            await this.assistantStatus(assistantId)
            .then(status => needTraining = status.needTraining)
            .catch(error => {
                throw new Error('NLU training error: ' + error.message)
            })
        }
    }

    public async asrTrainingStatus(assistantId: string): Promise<ASRTrainingStatus> {
        return this.wretch.url(`/v1/languagemodel/status`)
        .query({ assistantId })
        .get()
        .json(json => new ASRTrainingStatus(json))
    }

    public async asrTraining(assistantId: string, forceTraining = false) {
        let trainingStatus = 'notok'

        await this.asrTrainingStatus(assistantId)
        .then(asrStatus => trainingStatus = asrStatus.status)
        .catch(_ => {})

        if (forceTraining) trainingStatus = 'notok'
        if (trainingStatus === 'ok') return

        await this.wretch.url('/v1/languagemodel')
        .json({ assistantId })
        .post()
        .error(409, () => {})
        .res()
        .catch(_ => {})

        while (trainingStatus !== 'ok') {
            await timeout(5000)
            await this.asrTrainingStatus(assistantId)
            .then(asrStatus => trainingStatus = asrStatus.status)
            .catch(error => {
                throw new Error('Language model training error: ' + error.message)
            })
        }
    }

    public async downloadAssistant(assistantId: string, steps: (string) => void, forceNLUTraining = false, forceASRTraining = false): Promise<string> {
        steps('NLU training')
        await this.nluTraining(assistantId, forceNLUTraining)
        steps('ASR training')
        await this.asrTraining(assistantId, forceASRTraining)

        steps('Downloading assistant')
        return await this.wretch
        .url(`/v3/assistant/${assistantId}/download`)
        .get()
        .res(response => {
            const filePath = path.join(samConfigDirectory, 'assistant.zip')
            const file: NodeJS.WritableStream = fs.createWriteStream(filePath)
            return new Promise<string>((resolve, reject) => {
                if (response.body) {
                    response.body['pipe'](file)
                    .on('finish', () => {
                        resolve(filePath)
                    })
                    .on('error', () => {
                        reject('Error downloading assistant')
                    })
                } else {
                    reject('Response has no body')
                }
            })
        })
    }
}

==> snips-sam-0.0.47.tgz: package/src/session/consoleClient.ts <==
import { timeout, samConfigDirectory } from '../utils'
import wretch from 'wretch'
import { Wretcher } from 'wretch/dist/wretcher'
import { cli } from '../cli'
import { User, ConsoleToken, Assistant, AssistantStatus, ASRTrainingStatus } from '../models'

const path = require('path')
const fs = require('fs')
const uuidv4 = require('uuid/v4')

export class ConsoleClient {
    private wretch: Wretcher

    constructor() {
        // const httpsProxyAgent = require('https-proxy-agent')
        this.wretch = wretch()
        .url('https://external-gateway.snips.ai')
        // .options({ agent: httpsProxyAgent('http://127.0.0.1:8888') })
        .polyfills({
            fetch: require('node-fetch'),
            FormData: require('form-data'),
            URLSearchParams: require('url').URLSearchParams,
        })
        .catcher(401, _ => {
            throw new Error('You need to login again')
        })

        if (cli.config.consoleToken && cli.config.consoleToken.token !== undefined) {
            this.wretch.defaults({ headers: { Authorization: cli.config.consoleToken.token } })
        }
    }

    public async login(email: string, password: string): Promise<User> {
        return this.wretch
        .url('/v1/user/auth')
        .json({ email, password })
        .post()
        .res()
        .then(response => {
            const token = response.headers.get('authorization')
            if (token === null) throw new Error('Error fetching JWT token')
            return response.json()
            .then(json => {
                const user: User = new User(json['user'])
                return { user, token }
            })
        })
        .then(tuple => {
            return this.accessToken(tuple.user.id, tuple.token)
            .then(tokenObject => {
                const consoleCredentials = new ConsoleToken(tokenObject)
                consoleCredentials.token = 'JWT ' + consoleCredentials.token
                this.wretch.defaults({ headers: { Authorization: consoleCredentials.token } })
                return { user: tuple.user, token: consoleCredentials }
            })
        })
        .then(tuple => {
            const user: User = { id: tuple.user.id }
            cli.config.user = user
            cli.config.consoleToken = tuple.token
            return cli.config.save().then(_ => user)
        })
    }

    public async logout(userId: string, alias: string) {
        return this.wretch
        .url(`/v1/user/${userId}/accesstoken/${alias}`)
        .delete()
        .error(204, () => {})
        .res()
        .then(_ => {
            cli.config.user = undefined
            cli.config.consoleToken = undefined
            return cli.config.save()
        })
    }

    public async accessToken(userId: string, jwtToken: string): Promise<ConsoleToken> {
        const alias = (`sam-` + uuidv4()).replace(/-/g, '').substring(0, 29)
        return this.wretch
        .url(`/v1/user/${userId}/accesstoken`)
        .auth(jwtToken)
        .json({ alias })
        .post()
        .json(json => new ConsoleToken(json['token']))
    }

    public async assistantList(userId: string): Promise<Assistant[]> {
        return this.wretch
        .url('/v3/assistant')
        .query({ userId })
        .get()
        .json(json => {
            const assistants: Assistant[] = json['assistants'].map(assistant => Assistant.fromJSON(assistant))
            if (assistants === null || assistants === undefined) return []
            return assistants
        })
    }

    public async assistantStatus(assistantId: string): Promise<AssistantStatus> {
        return this.wretch
        .url(`/v3/assistant/${assistantId}/status`)
        .get()
        .json(json => new AssistantStatus(json))
    }

    public async nluTraining(assistantId: string, forceTraining = false) {
        let needTraining = false
        await this.assistantStatus(assistantId)
        .then(status => needTraining = status.needTraining)
        .catch(error => {
            if (error.statusCode === 409) { // Already training
                needTraining = false
            } else {
                throw new Error('NLU training error: ' + error.message)
            }
        })

        if (needTraining || forceTraining) {
            await this.wretch.url('/v1/training').json({ assistantId }).post()
        }

        while (needTraining) {
            await timeout(2000)
            await this.assistantStatus(assistantId)
            .then(status => needTraining = status.needTraining)
            .catch(error => {
                throw new Error('NLU training error: ' + error.message)
            })
        }
    }

    public async asrTrainingStatus(assistantId: string): Promise<ASRTrainingStatus> {
        return this.wretch.url(`/v1/languagemodel/status`)
        .query({ assistantId })
        .get()
        .json(json => new ASRTrainingStatus(json))
    }

    public async asrTraining(assistantId: string, forceTraining = false) {
        let trainingStatus = 'notok'

        await this.asrTrainingStatus(assistantId)
        .then(asrStatus => trainingStatus = asrStatus.status)
        .catch(_ => {})

        if (forceTraining) trainingStatus = 'notok'
        if (trainingStatus === 'ok') return

        await this.wretch.url('/v1/languagemodel')
        .json({ assistantId })
        .post()
        .error(409, () => {})
        .res()
        .catch(error => {
            throw new Error('Language model training error: ' + JSON.parse(error.message).message)
        })

        while (trainingStatus !== 'ok') {
            await timeout(5000)
            await this.asrTrainingStatus(assistantId)
            .then(asrStatus => trainingStatus = asrStatus.status)
            .catch(error => {
                throw new Error('Language model training error: ' + error.message)
            })
        }
    }

    public async downloadAssistant(assistantId: string, steps: (string) => void, forceNLUTraining = false, forceASRTraining = false): Promise<string> {
        steps('NLU training')
        return await this.nluTraining(assistantId, forceNLUTraining)
        .then(_ => steps('ASR training'))
        .then(_ => this.asrTraining(assistantId, forceASRTraining))
        .then(_ => steps('Downloading assistant'))
        .then(_ =>
            this.wretch
            .url(`/v3/assistant/${assistantId}/download`)
            .get()
            .res(response => {
                const filePath = path.join(samConfigDirectory, 'assistant.zip')
                const file: NodeJS.WritableStream = fs.createWriteStream(filePath)
                return new Promise<string>((resolve, reject) => {
                    if (response.body) {
                        response.body['pipe'](file)
                        .on('finish', () => {
                            resolve(filePath)
                        })
                        .on('error', () => {
                            reject('Error downloading assistant')
                        })
                    } else {
                        reject('Response has no body')
                    }
                })
            }),
        )
    }
}
